@@ -1,6 +1,6 @@
-/* sha3-224.c
-
-   The sha3 hash function, 224 bit output.
+/* sha3-256.c
+   
+   The sha3 hash function, 256 bit output.
 
    Copyright (C) 2012 Niels Möller
 
@@ -44,26 +44,27 @@
 #include "nettle-write.h"
 
 void
-sha3_224_init (struct sha3_224_ctx *ctx)
+sha3_256_init (struct sha3_256_ctx *ctx)
 {
-  memset (ctx, 0, offsetof (struct sha3_224_ctx, block));
+  memset (ctx, 0, offsetof (struct sha3_256_ctx, block));
 }
 
 void
-sha3_224_update (struct sha3_224_ctx *ctx,
+sha3_256_update (struct sha3_256_ctx *ctx,
 		 size_t length,
 		 const uint8_t *data)
 {
-  ctx->index = _sha3_update (&ctx->state, SHA3_224_BLOCK_SIZE, ctx->block,
-			     ctx->index, length, data);
+  ctx->index = _nettle_sha3_update (&ctx->state,
+				    SHA3_256_BLOCK_SIZE, ctx->block,
+				    ctx->index, length, data);
 }
 
 void
-sha3_224_digest(struct sha3_224_ctx *ctx,
+sha3_256_digest(struct sha3_256_ctx *ctx,
 		size_t length,
 		uint8_t *digest)
 {
-  _sha3_pad_hash (&ctx->state, SHA3_224_BLOCK_SIZE, ctx->block, ctx->index);
+  _sha3_pad_hash (&ctx->state, SHA3_256_BLOCK_SIZE, ctx->block, ctx->index);
   _nettle_write_le64 (length, digest, ctx->state.a);
-  sha3_224_init (ctx);
+  sha3_256_init (ctx);
 }
